# -*- coding: utf-8 -*-

import sys
import os
import requests
import concurrent.futures

"""
Set django environment
"""
sys.path.append(os.path.join(os.path.dirname(__file__), '../schoolsproject'))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "schoolsproject.settings.dev")

from schools import models
import django


def geocode_school(school):
    geocode_url = 'http://geocode-maps.yandex.ru/1.x/?format=json&results=1&geocode=город {0},{1}'
    r = requests.get(
        geocode_url.format(school.city.name, school.address)
    )
    r = r.json()
    try:
        point = r["response"]["GeoObjectCollection"]["featureMember"][0]["GeoObject"]["Point"]["pos"]
        print(school.name + " " + point)
        school.longitude, school.latitude = point.split()
        school.save()
    except KeyError:
        print(school.name + " Failed to get coordinates")


def concurrent_geocode():
    with concurrent.futures.ThreadPoolExecutor(max_workers=8) as executor:
        executor.map(geocode_school, models.School.objects.filter(city__alias='barnaul').exclude(address__isnull=True))


if __name__ == "__main__":
    django.setup()
    concurrent_geocode()