# -*- coding: utf-8 -*-

import sys
import os

# set config
sys.path.append(os.path.join(os.path.dirname(__file__), '../schoolsproject'))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "schoolsproject.settings.dev")

# import models
from schools import models
import django

django.setup()

from ftplib import FTP
import zipfile
import xml.etree.ElementTree as Et
import re

file_names = []
data = {}


def download(dirs, region):
    session = FTP('ftp.bus.gov.ru')  # connect to ftp-server
    session.login("gmuext", "YctTa34AdOPyld2")  # login to ftp-server

    for directory in dirs:
        session.cwd("/" + directory + "/" + region + "000000000/")  # change directory
        os.makedirs('files/' + directory, exist_ok=True)
        local_file_names = list(filter(lambda x: x.endswith(".xml.zip"), os.listdir('files/' + directory)))
        ftp_file_names = list(filter(lambda x: "all" in x and x.endswith(".xml.zip"), session.nlst()))  # make massive with filenames

        if local_file_names == ftp_file_names:
            file_names.append(local_file_names)
        else:
            for filename in local_file_names:
                os.remove('files/' + directory + "/" + filename)

            for filename in ftp_file_names:
                session.retrbinary("RETR " + filename, open("files/" + directory + "/" + filename, 'wb').write) #  download files
                print("Loading " + filename)

            file_names.append(ftp_file_names)

    session.close()


def scan():
    file_names.append(list(filter(lambda x: x.endswith('.zip'), os.listdir('files/GeneralInfo'))))
    file_names.append(list(filter(lambda x: x.endswith('.zip'), os.listdir('files/ActivityResult'))))
    file_names.append(list(filter(lambda x: x.endswith('.zip'), os.listdir('files/StateTask'))))


# scan()
download(['GeneralInfo', 'ActivityResult', 'StateTask'], "22")
print(file_names)


def simple_index_parser(town):
    for file_name in file_names[0]:
        print("Checking " + file_name)
        archive = zipfile.ZipFile("files/GeneralInfo/" + file_name, 'r')
        for i in archive.namelist():
            if i.split('.')[-1] == "xml":
                print("Checking " + i)
                root = Et.parse(archive.open(i)).getroot()

                city = root.find(".//{http://bus.gov.ru/types/1}city/{http://bus.gov.ru/types/1}name")
                inn = root.find(".//{http://bus.gov.ru/types/1}inn")
                okved = root.find(".//{http://bus.gov.ru/types/1}okved/{http://bus.gov.ru/types/1}code")

                if inn.text == "2221031000":
                    print("ОНА ТУТ!")

                if city is not None and town.lower() in city.text.lower() and ("общеобразовательное" in root.find(".//{http://bus.gov.ru/types/1}fullName").text.lower() or "80.21.2" in okved.text) and "муни" in root.find(".//{http://bus.gov.ru/types/1}okogu/{http://bus.gov.ru/types/1}name").text.lower():

                    data.update({inn.text: {}})

                    data[inn.text]["source"] = file_name + " > " + i + " | "

                    def simple_index_filter(name, dir):
                        index_value = root.find(dir)
                        if index_value is None:
                            data[inn.text][name] = ""
                        else:
                            data[inn.text][name] = index_value.text

                    # simple_index_filter("name", ".//{http://bus.gov.ru/types/1}shortName")

                    index_value_name = root.find(".//{http://bus.gov.ru/types/1}shortName")
                    if index_value_name is None:
                        data[inn.text]["name"] = "Н/И"
                    else:
                        school_number = re.findall('(\d+)', index_value_name.text)
                        index_value_name = index_value_name.text.replace('МБОУ ','')
                        print(index_value_name)
                        if "гимназия" in index_value_name.lower() and len(school_number) == 1 :
                            data[inn.text]["name"] = "Гимназия №" + school_number[0]
                        elif "лицей" in index_value_name.lower() and len(school_number) == 1 :
                            data[inn.text]["name"] = "Лицей №" + school_number[0]
                        elif ("ош" in index_value_name.lower() or "школа" in index_value_name.lower()) and len(school_number) == 1 :
                            data[inn.text]["name"] = "Школа №" + school_number[0]
                        else:
                            data[inn.text]["name"] = index_value_name

                    index_value_building = root.find(".//{http://bus.gov.ru/types/1}building")
                    if index_value_building is None:
                        data[inn.text]["building"] = "Н/И"
                    else:
                        index_value_building = index_value_building.text.replace('/НЕТ','')
                        index_value_building = index_value_building.replace('/','')
                        print(index_value_building)
                        data[inn.text]["building"] = index_value_building

                    index_value_village = root.find(".//{http://bus.gov.ru/types/1}locality/{http://bus.gov.ru/types/1}name")
                    if index_value_village is None:
                        data[inn.text]["village"] = ""
                    else:
                        print(index_value_village)
                        data[inn.text]["village"] = index_value_village.text + ", "

                    index_value_street = root.find(".//{http://bus.gov.ru/types/1}street/{http://bus.gov.ru/types/1}name")
                    if index_value_street is None:
                        data[inn.text]["street"] = ""
                    else:
                        print(index_value_street)
                        data[inn.text]["street"] = index_value_street.text + ", "


                    # simple_index_filter("street", ".//{http://bus.gov.ru/types/1}street/{http://bus.gov.ru/types/1}name")
                    # simple_index_filter("building", ".//{http://bus.gov.ru/types/1}building")
                    simple_index_filter("city", ".//{http://bus.gov.ru/types/1}city/{http://bus.gov.ru/types/1}name")


def budget_index_parser():
    for file_name in file_names[1]:
        print("Checking " + file_name)
        archive = zipfile.ZipFile("files/ActivityResult/" + file_name, 'r')
        for j in archive.namelist():
            if j.split('.')[-1] == "xml":
                print("Checking " + j)
                root = Et.parse(archive.open(j)).getroot()
                inn = root.find(".//{http://bus.gov.ru/types/1}inn")
                report_year = root.find(".//{http://bus.gov.ru/types/1}reportYear")
                if inn.text in data:

                    data[inn.text]["source"] = data[inn.text]["source"] + file_name + " > " + j + " | "

                    def budget_index_filter(name, dir):
                        index_value = root.find(dir)
                        if index_value is None or index_value.text == "0":
                            data[inn.text][name + report_year.text] = "Н/И"
                        else:
                            data[inn.text][name + report_year.text] = round(float(index_value.text), 0)

                    budget_index_filter("average_salary", ".//{http://bus.gov.ru/types/1}staff/{http://bus.gov.ru/types/1}averageSalary")
                    budget_index_filter("receipts", ".//{http://bus.gov.ru/types/1}cashReceipts/{http://bus.gov.ru/types/1}total")

                    all_payments = root.findall(".//{http://bus.gov.ru/types/1}cashPayments/{http://bus.gov.ru/types/1}budgetary/{http://bus.gov.ru/types/1}payment")
                    payments = sum(float(all_payments[i].text) for i in range(0, int(len(all_payments))))
                    if payments is not None and payments != 0:
                        data[inn.text]["payments" + report_year.text] = round(payments, 0)


def task_index_parser():
    for file_name in file_names[2]:
        print("Checking " + file_name)
        archive = zipfile.ZipFile("files/StateTask/" + file_name, 'r')
        for i in archive.namelist():
            if i.split('.')[-1] == "xml":
                print("Checking " + i)
                root = Et.parse(archive.open(i)).getroot()

                inn = root.find(".//{http://bus.gov.ru/types/1}inn").text
                report_year = root.find(".//{http://bus.gov.ru/types/1}reportYear")

                if inn in data and report_year is not None:

                    data[inn]["source"] = data[inn]["source"] + file_name + " > " + i + " | "

                    print(inn)
                    indexes = []
                    post_indexes = [[],[],[],[],[]]
                    print(root.find(".//{http://bus.gov.ru/types/1}fullName").text)

                    for index in root.findall('.//{http://bus.gov.ru/types/1}volumeIndex'):
                        indexes.append([index.find(".//{http://bus.gov.ru/types/1}index/{http://bus.gov.ru/types/1}name"), index.find(".//{http://bus.gov.ru/types/1}valueActual/{http://bus.gov.ru/types/1}actualValue"), index.find(".//{http://bus.gov.ru/types/1}symbol")])

                    for index in root.findall('.//{http://bus.gov.ru/types/1}qualityIndex'):
                        indexes.append([index.find(".//{http://bus.gov.ru/types/1}index/{http://bus.gov.ru/types/1}name"), index.find(".//{http://bus.gov.ru/types/1}valueActual/{http://bus.gov.ru/types/1}actualValue"), index.find(".//{http://bus.gov.ru/types/1}symbol")])

                    for index in indexes:
                        if index[1] is not None and index[2] is not None:

                            if ("ютер" in index[0].text.lower() or " комп" in index[0].text.lower()) and "педагог" not in index[0].text.lower():
                                print(index[0].text, index[1].text, index[2].text)
                                post_indexes[2].append(index[1].text)
                            elif ("чел" in index[2].text.lower() or "ед" in index[2].text.lower() or "уч" in index[2].text.lower()) and "класс" not in index[0].text.lower() and "комп" not in index[0].text.lower() and "сел" not in index[0].text.lower() and "план" not in index[0].text.lower():
                                print(index[0].text, index[1].text, index[2].text)
                                post_indexes[0].append(index[1].text)
                            elif "педагог" in index[0].text.lower() and "проф" in index[0].text.lower():
                                print(index[0].text, index[1].text)
                                post_indexes[1].append(index[1].text)
                            elif ("обучающи" in index[0].text.lower() or "учащи" in index[0].text.lower()) and ("чел" in index[2].text.lower() or "ед" in index[2].text.lower() or "уч" in index[2].text.lower()) and "напол" not in index[0].text.lower() and "комп" not in index[0].text.lower() and "сел" not in index[0].text.lower():
                                print(index[0].text, index[1].text, index[2].text)
                                post_indexes[0].append(index[1].text)
                            elif ("единый" in index[0].text.lower() or "егэ" in index[0].text.lower()) and "матем" not in index[0].text.lower():
                                print(index[0].text, index[1].text, index[2].text)
                                try:
                                    if float(index[1].text.replace(",",  ".")) > 20:
                                        post_indexes[3].append(index[1].text)
                                except:
                                    pass
                            elif "олимп" in index[0].text.lower():
                                print(index[0].text, index[1].text, index[2].text)
                                post_indexes[4].append(index[1].text)
                            else:
                                pass

                    if post_indexes[4] == ["0"] or post_indexes[4] == ["100"] == post_indexes[3]:
                        post_indexes[3] = []

                    if post_indexes[4] == ["0"] or post_indexes[4] == ["100"] == post_indexes[1]:
                        post_indexes[1] = []


                    print(post_indexes)

                    def relative_index(massive_of_indexes,index_name):

                        index_exit = 0

                        try:
                            if len(massive_of_indexes) == 1:
                                index_exit = round(float(massive_of_indexes[0].replace(",",  ".")),2)
                            elif len(massive_of_indexes) == 0:
                                pass
                            elif 1 < len(massive_of_indexes) < 4 and "-" not in massive_of_indexes[0]:
                                index_exit = round(sum(float(massive_of_indexes[i].replace(",",  ".")) for i in range(0, len(massive_of_indexes)))/len(massive_of_indexes), 2)
                            elif len(massive_of_indexes) == 4:
                                index_exit = float(massive_of_indexes[3].replace(",",  "."))

                            if index_exit != 0:
                                data[inn][index_name + report_year.text] = index_exit

                        except ValueError:
                            print("Ошибка в " + index_name + "! - ", massive_of_indexes)
                            pass

                    def absolute_index(massive_of_indexes,index_name):

                        index_exit = 0

                        try:
                            if 0 < len(massive_of_indexes) < 4:
                                if all(i == massive_of_indexes[0] and i != 0 for i in massive_of_indexes):
                                    index_exit = int(massive_of_indexes[0])
                                else:
                                    index_exit = round(sum(int(massive_of_indexes[i]) for i in range(0, len(massive_of_indexes))), 2)
                            elif len(massive_of_indexes) == 4:
                                index_exit = int(massive_of_indexes[3])
                            elif len(massive_of_indexes) == 0:
                                pass

                            if index_exit != 0:
                                data[inn][index_name + report_year.text] = index_exit

                        except ValueError:
                            print("Ошибка в " + index_name + "! - ", massive_of_indexes)
                            pass

                    relative_index(post_indexes[1], "high_education_index")
                    absolute_index(post_indexes[0], "number_of_students_index")
                    relative_index(post_indexes[2], "computer_index")
                    relative_index(post_indexes[3], "ege_index")
                    print(data[inn])


simple_index_parser("Барнаул")
budget_index_parser()
task_index_parser()

print(len(data))
print(data)

models.School.objects.all().delete()
models.City.objects.all().delete()

city = models.City(name="Барнаул", alias="barnaul")
city.save()

for i in data.values():
    print(i["name"])
    school = models.School(name=i["name"], city=city, address=i["village"]+i["street"]+i["building"], source=i["source"])

    try:
        school.payments = i["payments2013"]
    except KeyError:
        pass

    try:

        school.ege_index = i["ege_index2014"]
    except KeyError:
        try:
            school.ege_index = i["ege_index2013"]
        except KeyError:
            try:
                school.ege_index = i["ege_index2012"]
            except KeyError:
                try:
                    school.ege_index = i["ege_index2011"]
                except KeyError:
                    pass


    try:
        school.receipts = i["receipts2013"]
    except KeyError:
        pass

    try:
        school.average_salary = i["average_salary2013"]
    except KeyError:
        pass

    try:
        school.number_of_students_index = i["number_of_students_index2014"]
    except KeyError:
        try:
            school.number_of_students_index = i["number_of_students_index2013"]
        except KeyError:
            try:
                school.number_of_students_index = i["number_of_students_index2012"]
            except KeyError:
                try:
                    school.number_of_students_index = i["number_of_students_index2011"]
                except KeyError:
                    pass

    try:
        school.computer_index = i["computer_index2014"]
    except KeyError:
        try:
            school.computer_index = i["computer_index2013"]
        except KeyError:
            try:
                school.computer_index = i["computer_index2012"]
            except KeyError:
                try:
                    school.computer_index = i["computer_index2011"]
                except KeyError:
                    pass

    try:
        school.high_education_index = i["high_education_index2014"]
    except KeyError:
        try:
            school.high_education_index = i["high_education_index2013"]
        except KeyError:
            try:
                school.high_education_index = i["high_education_index2012"]
            except KeyError:
                try:
                    school.high_education_index = i["high_education_index2011"]
                except KeyError:
                    pass



    school.save()






"""
Add cities & schools:

from schools.models import models
city = models.City(name="Барнаул")
city.save()

school = models.School(name="74 school", city=city, address="Smirnova 92")
school.save()


Add fields to models:

1. add field with type https://docs.djangoproject.com/en/1.7/ref/models/fields/
2. python manage.py makemigrations OR Tools -> Run manage.py task -> "makemigrations"
3. python manage.py migrate OR Tools -> Run manage.py task -> "migrate"
"""
