from django.db import models


class City(models.Model):
    name = models.CharField(max_length=100, help_text="City name")
    alias = models.CharField(max_length=30, default="", help_text="City alias")
    average_receipts = models.FloatField(blank=True, null=True, default=None, help_text="City average schools receipts")
    average_high_education_index = models.FloatField(blank=True, null=True, default=None,
                                                     help_text="City average high education index")
    average_ege_index = models.FloatField(blank=True, null=True, default=None, help_text="City average ege index")
    average_computer_index = models.FloatField(blank=True, null=True, default=None,
                                               help_text="City average computer index")
    average_salary = models.FloatField(blank=True, null=True, default=None, help_text="City average salary")
    average_payments_per_student = models.FloatField(blank=True, null=True, default=None,
                                                     help_text="City average payments per student")

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "Cities"


class School(models.Model):
    name = models.CharField(max_length=300, help_text="School name")
    city = models.ForeignKey(City, help_text="School city")
    address = models.CharField(max_length=300, help_text="School address")
    inn = models.CharField(default=0, max_length=10, help_text="School inn")
    average_salary = models.FloatField(null=True, default=None, help_text="School Average Salary")
    receipts = models.FloatField(null=True, default=None, help_text="School Receipts")
    payments = models.FloatField(null=True, default=None, help_text="School Payments")
    high_education_index = models.FloatField(null=True, default=None, help_text="School High Education Index")
    number_of_students_index = models.FloatField(null=True, default=None, help_text="School number of students")
    computer_index = models.FloatField(null=True, default=None, help_text="School number of student per computer")
    payments_per_student = models.FloatField(null=True, default=None, help_text="Payments per student")
    ege_index = models.FloatField(null=True, default=None, help_text="EGE")
    longitude = models.FloatField(null=True, default=None, help_text="longitude")
    latitude = models.FloatField(null=True, default=None, help_text="latitude")
    source = models.TextField(help_text="source", default="")


    def __str__(self):
        return self.name
