(function(window) {

	window.store = {

		/**
		 * Schools for compare list
		 */
		comparesSchoolsList: [],

		/**
		 * Add school to compared schools list
		 * @param school {object}
		 */
		addSchoolToComparedList: function (school) {
			if (this.isSchoolInComparedList(school)) {
				return;
			}
			this.comparesSchoolsList.push(school);
			this.saveComparedSchoolsList();
		},

		/**
		 * Remove school from compared schools list
		 * @param school {object}
		 */
		removeSchoolFromComparedList2: function (school) {
			this.comparesSchoolsList = this.comparesSchoolsList.filter(function(schoolItem) {
				return schoolItem.id !== school.id;
			});
			this.saveComparedSchoolsList();
		},

		/**
		 * Is school in compared schools list?
		 * @param school {object}
		 * @returns {boolean}
		 */
		isSchoolInComparedList: function (school) {
			for (var i = 0; i < this.comparesSchoolsList.length; i++) {
				if (this.comparesSchoolsList[i].id === school.id) {
					return true;
				}
			}
			return false;
		},

		/**
		 * Save compared schools list
		 */
		saveComparedSchoolsList: function () {
			window.localStorage.setItem('comparedSchoolsList', JSON.stringify(this.comparesSchoolsList));
		},

		/**
		 * Get compared schools list
		 * @returns {object[]}
		 */
		getComparedSchoolsList: function () {
			if (window.localStorage.getItem('comparedSchoolsList')) {
				this.comparesSchoolsList = JSON.parse(window.localStorage.getItem('comparedSchoolsList'));
			}
			return this.comparesSchoolsList;
		},

		/**
		 * Get compare schools list length
		 * @returns {number}
		 */
		getComparedSchoolsListLength: function () {
			return this.getComparedSchoolsList().length;
		},

		/**
		 * Get schools grouped by parameter from max to min [3 top schools, 3 bottom schools]
		 * @param parameter {string}
		 * @param success {function}
		 */
		getSchoolsByParameter: function (parameter, success) {
			$.get('/short/' + parameter, function (res) {
				success(res);
			});
		},

		/**
		 * Get all schools by parameter from max to min
		 * @param parameter {string}
		 * @param success {function}
		 */
		getAllSchoolsByParameter: function (parameter, success) {
			$.get('/' + parameter, function (res) {
				success(res);
			});
		},

		/**
		 * Get schools with coordinates
		 * @param success {function}
		 */
		getSchoolsWithCoordinates : function (success) {
			$.get('/coordinates', function (res) {
				success(res);
			});
		},

		/**
		 * Get school info by id
		 * @param id {number|string}
		 * @param success {function}
		 */
		getSchoolInfo: function (id, success) {
			$.get('/school/' + id, function (res) {
				success(res);
			});
		}
	};

})(window);